
# This is the grammar for grammar files.
#
# Which means you can use the Pattern returned by compile()'ing this file
# to parse this file.
#
# So... this file serves at the same time as both an example source text
# and the grammar to parse it.
#
# I realize that's possibly more confusing than helpful, so I'll include
# another simpler example elsewhere.  :)
#
# To "test" this file, run esrapy.py directly (i.e., "python esrapy.py").


# Comment-sucking optional white space with indentation:
pSpace = <([\r\n]*([ \t]+|[ \t]*#[^\n\r]*))*>

# Comment-sucking white space to (and including) end of line (or end of source):
pEndLine  = <([ \t]*(#[^\r\n]*)?([\n\r]|\Z))+>
# Optional version of same:
pEndLine0 = <([ \t]*(#[^\r\n]*)?([\n\r]|\Z))*>

# Variable name, and integer:
pVarname = <[a-zA-Z_]+[a-zA-Z_0-9]*>
pInt     = <[0-9]+>

pBaseSep = base:pBase@2 sep : (pSpace '/' pSpace _:pBase@2 ) ?

# The base set of patterns:
pBase = 0,set; (attrs:(_:(pInt|pVarname)/(pSpace ',' pSpace){1,2} pSpace ';' pSpace)?
				pat:pBase@1)/(pSpace '|' pSpace){2}
	  | 1,seq; (name:(_:pVarname pSpace ':' pSpace)? pat:pBase@2)/pSpace{2}
	  | 2,mod; base:pBaseSep pSpace op:<[?*+]>
	  | range; base:pBaseSep pSpace '{' pSpace range:pInt/(pSpace ',' pSpace){1,2} pSpace '}'
	  |   var; pVarname
	  |   key; '\' pSpace _:pVarname
	  |subset; var:pVarname pSpace '@' pSpace limit:pInt
	  | regex; expr:(`<([^>]*)>`@1 | <`([^`]*)`>@1) group:(pSpace '@' pSpace _:pInt)?
	  |string; (<"([^"]*)">@1 | <'([^']*)'>@1)
	  | '(' pSpace _:pBase pSpace ')'

# And the top level operation, assigning a pattern to a variable:
pAssign = var:pVarname pSpace '=' pSpace val:pBase pEndLine

# The assign includes trailing white space, but we still need to allow for heading:
pFile = pEndLine0 _:pAssign*

