
import esrapy

# Here's a Pattern that parses expressions like "a = b*5+(2^c)":
pattern = esrapy.compile(r"""
SPACE = <[ \t]*>
expr  = name:<[a-zA-Z_]+> op:'=' b:expr
		|1; a:expr@1 op:<[+-]> b:expr@2
		|2; a:expr@2 op:<[*/]> b:expr@3
		|3; a:expr@3 op:'^'    b:expr@4
		|4; name:<[a-zA-Z_]+> op:\var
		|    val:<[0-9]+>     op:\int
		| '(' _:expr ')'
""")

# And here's code that evaluates expressions like that...
vars = {}
def eval_pt(pt):
	if pt.op == 'int': return int(pt.val)
	if pt.op ==   '+': return eval_pt(pt.a)  + eval_pt(pt.b)
	if pt.op ==   '-': return eval_pt(pt.a)  - eval_pt(pt.b)
	if pt.op ==   '*': return eval_pt(pt.a)  * eval_pt(pt.b)
	if pt.op ==   '/': return eval_pt(pt.a)  / eval_pt(pt.b)
	if pt.op ==   '^': return eval_pt(pt.a) ** eval_pt(pt.b)
	if pt.op == 'var': return vars[pt.name]
	if pt.op ==   '=':
		val = eval_pt(pt.b)
		vars[pt.name] = val
		return val
	raise "Unhandled op: %s"%pt.op

while True:
	try:
		s = raw_input("Expr: ")
	except:
		break
	try:
		pt = pattern.match(s, debug=1)
	except:
		print "Sorry, couldn't parse that."
		continue
	print "Value:", eval_pt(pt)

